/** Default API Error response */
export interface ErrorResponse {
    /** Error message */
    err: string;
    /** Error code */
    errCode: string;
}
/** Create shorten link */
export interface ShortenerInput {
    url: string;
    metadata?: object | null;
}
/** Shorten link response */
export interface ShortenerResponse {
    url: string;
}
/** Shorten link detailed response */
export interface ShortenerDetailResponse {
    link: {
        link: string;
        linkHash: string;
        shortHash: string;
        clickCount?: number | null;
        metadata?: object | null;
        createdOn: string;
    }[];
    clicks: {
        linkHash: string;
        shortHash: string;
        metadata?: object | null;
        createdOn: string;
    }[];
}
export type ListHealthData = any;
export type ListHealthError = ErrorResponse;
export type ListShortenData = ShortenerResponse;
export type ListShortenError = ErrorResponse;
export type GetCodeError = ErrorResponse;
export type GetDetailData = ShortenerDetailResponse;
export type GetDetailError = ErrorResponse;
export type GetRootError = ErrorResponse;
import type { AxiosInstance, AxiosRequestConfig, ResponseType } from 'axios';
export type QueryParamsType = Record<string | number, any>;
export interface FullRequestParams extends Omit<AxiosRequestConfig, 'data' | 'params' | 'url' | 'responseType'> {
    /** set parameter to `true` for call `securityWorker` for this request */
    secure?: boolean;
    /** request path */
    path: string;
    /** content type of request body */
    type?: ContentType;
    /** query params */
    query?: QueryParamsType;
    /** format of response (i.e. response.json() -> format: "json") */
    format?: ResponseType;
    /** request body */
    body?: unknown;
}
export type RequestParams = Omit<FullRequestParams, 'body' | 'method' | 'query' | 'path'>;
export interface ApiConfig<SecurityDataType = unknown> extends Omit<AxiosRequestConfig, 'data' | 'cancelToken'> {
    securityWorker?: (securityData: SecurityDataType | null) => Promise<AxiosRequestConfig | void> | AxiosRequestConfig | void;
    secure?: boolean;
    format?: ResponseType;
}
export declare enum ContentType {
    Json = "application/json",
    FormData = "multipart/form-data",
    UrlEncoded = "application/x-www-form-urlencoded",
    Text = "text/plain"
}
export declare class HttpClient<SecurityDataType = unknown> {
    instance: AxiosInstance;
    private securityData;
    private securityWorker?;
    private secure?;
    private format?;
    constructor({ securityWorker, secure, format, ...axiosConfig }?: ApiConfig<SecurityDataType>);
    setSecurityData: (data: SecurityDataType | null) => void;
    protected mergeRequestParams(params1: AxiosRequestConfig, params2?: AxiosRequestConfig): AxiosRequestConfig;
    protected stringifyFormItem(formItem: unknown): string;
    protected createFormData(input: Record<string, unknown>): FormData;
    request: <T = any, _E = any>({ secure, path, type, query, format, body, ...params }: FullRequestParams) => Promise<T>;
}
/**
 * @title Example microservice
 * @version 1.0.3
 * @baseUrl http://localhost:8080
 * @contact Example Company <someone@example.com> (http://example.com)
 */
export declare class Api<SecurityDataType extends unknown> extends HttpClient<SecurityDataType> {
    /**
     * @description Resolves a shortened link if found
     *
     * @tags Shorten
     * @name GetRoot
     * @summary Resolves link
     * @request GET:/
     */
    getRoot: (params?: RequestParams) => Promise<any>;
    health: {
        /**
         * @description Returns if system is health or not
         *
         * @tags Health
         * @name ListHealth
         * @summary System health check
         * @request GET:/health
         */
        listHealth: (params?: RequestParams) => Promise<any>;
    };
    shorten: {
        /**
         * @description Shorten an link
         *
         * @tags Shorten
         * @name ListShorten
         * @summary Shorten an link
         * @request POST:/shorten
         * @secure
         */
        listShorten: (data: ShortenerInput, params?: RequestParams) => Promise<ShortenerResponse>;
    };
    code: {
        /**
         * @description Resolves a shortened link if found
         *
         * @tags Shorten
         * @name GetCode
         * @summary Resolves link
         * @request GET:/{code}
         */
        getCode: (code: string, params?: RequestParams) => Promise<any>;
        /**
         * @description Resolves a shortened link if found
         *
         * @tags Shorten
         * @name GetDetail
         * @summary Resolves link
         * @request GET:/{code}/detail
         * @secure
         */
        getDetail: (code: string, params?: RequestParams) => Promise<ShortenerDetailResponse>;
    };
}
